/*
	$Id: resource.h,v 1.20 2002/07/16 12:39:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="Resources"
//! header=core.h

#ifndef header_resource
#define header_resource

#include "../../signals.h"
#include <string>

class CL_ResourceOptions;
class CL_ResourceManager;
class CL_ResourceData;
class CL_InputSource;
class CL_OutputSource;
class CL_InputSourceProvider;
class CL_OutputSourceProvider;
class CL_Resource_Generic;

//: Interface for accessing a resource in the resource manager.
class CL_Resource
{
//! Construction:
public:
	//: Resource constructor.
	//param type: Resource type ('surface', 'font', etc).
	//param name: Name of resource ('Images/sur_image')
	//param location: Location field of resource ('image.jpg').
	//param options: Resource options of this resource.
	//param manager: Resource manager owning this resource.
	CL_Resource(
		const std::string &type,
		const std::string &name,
		const std::string &location,
		const CL_ResourceOptions &options,
		const CL_ResourceManager &manager);

	CL_Resource(const CL_Resource &copy);

	CL_Resource();

	//: Resource destructor.
	virtual ~CL_Resource();

//! Attributes:
public:
	//: Returns the type of the resource.
	const std::string &get_type() const;

	//: Returns the name of the resource.
	const std::string &get_name() const;

	//: Returns the location of the resource (relative to resource definition file).
	//- <p>Use this function if file is to be opened with CL_ResourceManager::get_resource_provider().</p>
	const std::string &get_location() const;

	//: Returns the full path location of the resource.
	//- <p>Use this function if file is to be opened without ClanLib input sources.</p>
	std::string get_full_location() const;

	//: Returns the resource options.
	CL_ResourceOptions &get_options();

	//: Returns the resource manager.
	CL_ResourceManager get_manager();

	//: Returns the data with the matching name.
	CL_ResourceData *get_data(const std::string &name);

	//: Returns the current reference count.
	int get_reference_count() const;

//! Signals:
public:
	//: Signal invoked when a call to load_file is made.
	CL_Signal_v0 &sig_load_file();

	//: Signal invoked when a call to load_datafile is made.
	CL_Signal_v1<CL_InputSourceProvider *> &sig_load_datafile();

	//: Signal invoked when a call to save_datafile is made.
	CL_Signal_v1<CL_OutputSourceProvider *> &sig_save_datafile();

	//: Signal invoked when a call to unload is made.
	CL_Signal_v0 &sig_unload();

//! Operators:
public:
	//: Copy a resource.
	void operator = (const CL_Resource &copy);

//! Operations:
public:
	//: Attach some data to the resource.
	void attach_data(const std::string &name, CL_ResourceData *data);

	//: Detach some data from the resource.
	void detach_data(CL_ResourceData *data);

	//: Unloads the resource from memory.
	void unload();

	//: Loads the resource, using the prefered source as specified by
	//: the resource manager.
	void load();

	//: Loads the resource from file.
	void load_file();

	//: Loads the resource from the input source.
	void load_datafile(CL_InputSourceProvider *input_provider);

	//: Saves the resource to the output source.
	void save_datafile(CL_OutputSourceProvider *output_provider);

//! Implementation:
public:
	CL_Resource(class CL_Resource_Generic *impl);

	//: Pointer to implementation
	CL_Resource_Generic *impl;
};

#endif
