/*
	$Id: resource_manager.h,v 1.23 2003/02/13 17:56:19 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="Resources"
//! header=core.h

#ifndef header_resource_manager
#define header_resource_manager

#include <list>
#include <string>

class CL_Resource;
class CL_InputSourceProvider;
class CL_ResourceManager_File;

class CL_ResourceManager
//: The ClanLib resource manager.
//- <p>The resource manager is used to retrieve resources from a given
//- resource source.</p>
//-
//- <p>This can either be a resource script file (used as input to the
//- datafile compiler), or a datafile with all the resources included into
//- one large gzipped resource file.</p>
//-
//- <p>To speedup loading of resources in a game, you can load entire sections
//- of resources at once. When a resource in the section is requested, it is
//- returned instantly without having to access the disk. This is especially
//- useful to make sure all the game resources are loaded before the game is
//- started.</p>
//-
//- <p>Resources are normally not retrieved using the get_resource() function.
//- Instead, you should load the resource using the appropiate resource type
//- class. For instance, a surface is easiest loaded like this:</p>
//-
//- <pre>
//- CL_ResourceManager res_manager("my_datafile.dat", true);
//- CL_Surface my_surface("my_surface", &res_manager);
//- </pre>
//-
//- <p>Getting the same resource twice won't create a new instance of the
//- resource; they are reference counted.</p>
{
//! Construction:
public:
	//: Resource Manager constructor.
	//param config_file: the name of the file in which the resources are defined
	//param provider: the optional inputprovider in which, the resource file is stored
	//param read_directly_from_source: if true, any resources are loaded directly from their source, meaning that any datafile-directive is ignored.
	//param file_name: the name of the file to open
	//param is_datafile: indicates if the file is a scriptfile or a datafile
	//param additional_resources: additional resources to be included into the resource set.
	CL_ResourceManager(
		const std::string &config_file,
		CL_InputSourceProvider *provider = NULL,
		bool read_directly_from_source=false,
		bool delete_inputsource_provider=false);

	CL_ResourceManager(
		const std::string &config_file,
		const bool is_datafile );

	CL_ResourceManager(
		const std::string &config_file,
		const bool is_datafile,
		CL_ResourceManager &additional_resources);

	CL_ResourceManager(const CL_ResourceManager &copy);

	CL_ResourceManager();

	//: Resource Manager destructor.
	~CL_ResourceManager();

//! Attributes:
public:
	//: Returns true if a resource exists.
	bool exists(const std::string &res_id);
	
	//: Returns a pointer to the CL_Resource representing the given resource.
	CL_Resource &get_resource(const std::string &res_id);

	//: Returns a list of all resources available.
	//- <p>Primarily used by the datafile compiler to build datafiles from resources.</p>
	//return: The list of resources available, in the form [section/subsection/.../]resourcename. You'll have to delete the list returned.
	std::list<std::string> *get_all_resources();

	//: Returns a list of all resources available matching a given type. 
	//- <p>Primarily used by the ClanCompiler to build datafiles from resources.</p>
	//return: The list of resources available. You'll have to delete the list returned.
	std::list<std::string> *get_resources_of_type(const std::string &type_id);

	//: Returns a pointer to the input source provider, in which all resources are stored.
	//- <p>This can be a file provider or a datafile provider depending
	//- on method used to load the script file.</p>
	//return: Pointer to inputsourceprovider containing resource data.
	CL_InputSourceProvider *get_resource_provider() const;

	//: Returns true if the resources are loaded from source. False if loaded from a datafile.
	bool is_from_source();

//! Operations:
public:
	//: Copy a resource manager.
	void operator = (const CL_ResourceManager &copy);

	//: Add resources from an other resource manager.
	void add_resources(const CL_ResourceManager &additional_resources);

	//: Remove resources from an other resource manager.
	void remove_resources(const CL_ResourceManager &additional_resources);

	//: Loads all resources into memory.
	void load_all();

	//: Unloads all resources from memory.
	void unload_all();

	//: Loads all resources in a given section into memory.
	void load_section(const std::string &section_name);

	//: Unloads all resources in a given section into memory.
	void unload_section(const std::string &section_name);

//! Implementation:
public:
	CL_ResourceManager(class CL_ResourceManager_File *impl);

	//: Pointer to the implementation
	CL_ResourceManager_File *impl;
};
#endif
