/*
	$Id: font.h,v 1.37 2003/02/05 04:38:24 plasmoid Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanDisplay="Fonts"
//! header=display.h

#ifndef header_font
#define header_font

#ifdef WIN32
#pragma warning( disable : 4786)
#endif

#include <string>
#include <utility> //For std::pair

#include "../Core/Math/origin.h"
#include "blend_func.h"
#include "../Core/Math/rect.h"
#include "../Core/Math/point.h"
#include "../Core/Math/size.h"
#include "../Core/Resources/resource.h"

class CL_GlyphBuffer;
class CL_Sprite;
class CL_TextStyler;
class CL_Font_Generic;
class CL_GraphicContext;
class CL_ResourceManager;

//: Draws text in bitmap and system fonts.
//- <p>A font can be either constructed from bitmap or from system
//- font (or loaded from a resource as either).</p>
//- <p>A bitmap font uses a CL_Sprite as the source for the font
//- glyphs (letters), where each frame frame in the sprite
//- represents one glyph. A string (<i>letter_chars</i>) is then
//- describing which character each glyph corresponds to. If the
//- sprite contains the letters ABCZXY123 in that order, then the
//- string should be "ABCZXY123".</p>
//- <p>To cut out letters for a bitmap font efficiently, have a look
//- at the alpha clipper in CL_SpriteDescription.</p>
//- <p>A system font uses the underlaying windowing system to create
//- the font glyphs. This means that in Windows you can choose any
//- TTF font, and same applies to X11 if the font server supports
//- it.</p>
//- <p>When a font is constructed based on an existing sprite, the
//- sprite is copied into the font, and several properties are changed.
//- Many properties of the font directly change the sprite's properties:
//- the functions that do this are marked 'This property is set inside
//- the glyphs sprite' in the documentation. In addition, when the sprite
//- is immediately loaded, its translation and rotation hotspots are
//- set to origin_top_left:0:0. The translation hotspot of the sprite
//- is important to positioning in general, and cannot be changed
//- within the font's internal sprite by the user. If you need to change it
//- you can always do your own drawing by copying the sprite out using
//- the get_glyphs() method. The rotation
//- hotspot inside the font's internal sprite can be set directly using
//- CL_Font::set_glyph_rotation_hotspot(), but at the cost of making
//- total rotation (i.e. changes to the CL_Font's angle) produce illegible though
//- still vaguely valid results. </p>
//- <p>Like CL_Surface and CL_Sprite, rotation does not affect
//- the calcluation of any bounding rectangles (such as the result returned
//- by draw() or bounding_rect(), or the rectangle calculated internally by
//- draw() for alignment). However, the height and width offset properties
//- still affect these calcluations, and so does the scale (since scaling
//- the CL_Font is effectively just changing the point size of the glyphs,
//- and that affects all sorts of things, such as word wrapping.)</p>
class CL_Font
{
//! Construction:
public:
	//: Constructs a font.
	//- <p>If spacelen is unspecified, CL_Font attempts to resolve
	//- it itself. First, it looks to see if you have specified a
	//- space character in letter_chars: if you have, it uses that
	//- glyph's width as the space width (as well as using it
	//- to actually draw the space). If it does not find a space
	//- glyph, then it will look at every glyph's width and make
	//- the space width the average of that.</p>
	//param resource_id: Font resource name.
	//param manager: Resource manager used to load font.
	//param glyphs: CL_Sprite containing the letters of a bitmap font.
	//param letters: String mapping each frame of the bitmap font sprite to letters.
	//param spacelen: Width in pixels of the space glyph.
	//param width_offset: Offset to apply to each glyph's width
	//param height_offset: Offset to apply to each glyph's height
	//param baseline: The initial baseline distance
	//param font_name: System font name (eg. "Arial").
	//param letter_chars: The characters to include in the font (if a sprite font, must be in frame order).
	//param height: Height of font in pixels.
	//param width: Width of font in pixels. If 0, uses best fitting width for the specified height.
	//param bold: If true, will use bold font.
	//param italic: If true, will use italic font.
	//param underline: If true, will use underlined font.
	//param strikeout: If true, will use striked out font.
	CL_Font();

	CL_Font(const CL_Font &copy);

	CL_Font(
		const std::string &resource_id,
		CL_ResourceManager *manager);
	
	CL_Font(
		const CL_Sprite &glyphs,
		const std::string &letters,
		int spacelen = -1,
		int width_offset = 0,
		int height_offset = 0,
		int baseline = 0);

	CL_Font(
		const std::string &font_name,
		const std::string &letters,
		int height,
		int width = 0,
		bool bold = false,
		bool italic = false,
		bool underline = false,
		bool strikeout = false);

	CL_Font(
		const std::string &font_name,
		int height,
		int width = 0,
		bool bold = false,
		bool italic = false,
		bool underline = false,
		bool strikeout = false);

	virtual ~CL_Font();

//! Subclasses:
public:
	//: The return type for draw(), draw_to_gb(), and test_draw().
	struct PrintInfo
	{
		//: A rectangle within which all glyphs were drawn.
		//- <p> The scale and rotation properties of the CL_Font
		//- don't affect this rectangle.</p>
		CL_Rect bounds;
		
		//: The upper-left position of where the next glyph would've been drawn.
		//- <p> The scale and rotation properties of the CL_Font
		//- don't affect this value.</p>
		CL_Point cursor;
		
		//: The last character in the string drawn, exclusive.
		//- <p> If you passed in a string literal to draw, then
		//- the string this iterator points into is already
		//- gone, and the iterator's invalid.</p>
		std::string::const_iterator end;
	};

//! Attributes:
public:
	//: Returns width offset.
	//- <p> The width offset can be used to kern glyphs together or spread them apart. </p> 
	int get_width_offset() const;
	
	//: Returns height offset.
	//- <p> The height offset can be used to create space between lines, or to merge them together. </p> 
	int get_height_offset() const;
	
	//: Returns baseline.
	//- <p> The baseline determines the bottom of the font in cases where multiple fonts appear on one line.
	//- The bottom, in this case, excludes the bottoms of glyphs that stretch below the line, normally
	//- including characters such as lowercase 'g' and 'j'. Returned value is the location of the baseline
	//- as number of pixels up from the bottom. </p>
	int get_baseline() const;

	//: Returns current scale.
	//- <p> 1.0f is normal scale, 2.0f is twice the size, etc. </p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void get_scale(float &x, float &y) const;

	//: Returns current alpha.
	//- <p> 0.0f is full transparency, and 1.0f is full visibility. </p>
	//- <p> This property is set inside the glyphs sprite.</p>
	float get_alpha() const;

	//: Returns current color.
	//- <p> Alpha 0.0f is full transparency, and 1.0f is full visibility (solid). </p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void get_color(float &red, float &green, float &blue, float &alpha) const;

	//: Returns blending functions.
	//- <p> This property is set inside the glyphs sprite.</p>
	void get_blend_func(CL_BlendFunc &src, CL_BlendFunc &dest) const;
	
	//: Returns angle of glyph rotation.
	//- <p> This rotation setting affects each individual glyph, rather
	//- then a drawn text as a whole. The glyphs are merely rotated
	//- from their original positions, useful for spinny effects, but also usually
	//- making things unreadable.</p>
	//- <p> This property is set inside the glyphs sprite.</p>
	float get_glyph_angle() const;
	
	//: Returns glyph rotation hotspot.
	//- <p> This is the hotspot origin which determines the rotation hotspot
	//- for individual rotation of glyphs (see CL_Font::set_glyph_angle()).
	//- Total rotation (not the same as glyph rotation) of the resulting text depends on this hotspot being
	//- origin_top_left:0:0, so if you set it to anything else, the results
	//- of glyph rotation combined with total rotation are unreadable (though still
	//- vaguely valid looking).
	//- This is fine if all you're doing is something like an explosion
	//- effect.</p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void get_glyph_rotation_hotspot(CL_Origin &origin, int &x, int &y) const;
	
	//: Returns angle in degrees of total rotation.
	//- <p> This only has an effect if you're using CL_Font's drawing mechanism itself.
	//- If the font is just being used inside a complex multi-font class (such as
	//- CL_TextStyler) then it makes no difference. If that's the case, use the 
	//- equivalent property of the class containing the font. </p>
	//- <p> Also note that rotation using this method with a glyph rotation hotspot
	//- besides origin_top_left:0:0 produces illegible results. </p>
	float get_angle() const;
	
	//: Returns the rotation hotspot.
	//- <p> This only has an effect if you're using CL_Font's drawing mechanism itself.
	//- If the font is just being used inside a complex multi-font class (such as
	//- CL_TextStyler) then it makes no difference. If that's the case, use the 
	//- equivalent property of the class containing the font. </p>
	void get_rotation_hotspot(CL_Origin &origin, int &x, int &y) const;
	
	//: Returns translation hotspot.
	//- <p> This only has an effect if you're using CL_Font's drawing mechanism itself.
	//- If the font is just being used inside a complex multi-font class (such as
	//- CL_TextStyler) then it makes no difference. If that's the case, use the 
	//- equivalent property of the class containing the CL_Font. </p>
	//- <p> The origin also serves as line justification, as with a word processor.
	//- That is, origin_*_center results in center justification, and so on. </p>
	void get_alignment(CL_Origin &origin, int &x, int &y) const;

	//: Returns the drawn height of the entire font or a string.
	//return: The height in pixels.
	//param str: String to get the height of.
	//param start: A starting iterator, inclusive.
	//param end: An ending iterator, exclusive.
	//param max_size: Same effect as the size of the dest rectangle passed to draw(), for word wrapping and height truncating
	//param delim: When word-wrapping, specifies characters (other than newlines) that act as word seperators.
	//- <p> The height of the entire font is the height of the tallest glyph in the font; this
	//- is what is returned if you specify no arguments. The point of having the height functions
	//- accept a string is to compensate for strings spanning multiple lines.</p>
	int get_height() const;
	
	int get_height(
		const std::string &str,
		const CL_Size &max_size = CL_Size(0,0),
		const std::string &delim = std::string(" ")) const;
	
	int get_height(
		const std::string::const_iterator &start,
		const std::string::const_iterator &end,
		const CL_Size &max_size = CL_Size(0,0),
		const std::string &delim = std::string(" ")) const;
	
	//: Returns the drawn width of a character or string.
	//return: The width in pixels.
	//param letter: Character to get the width of.
	//param str: String to get the width of.
	//param start: A starting iterator, inclusive.
	//param end: An ending iterator, exclusive.
	//param max_size: Same effect as the size of the dest rectangle passed to draw(), for word wrapping and height truncating
	//param delim: When word-wrapping, specifies characters (other than newlines) that act as word seperators.
	//- <p>It's important to note that newlines have no width, unless you
	//- specify a glyph for them in the sprite (which is pretty useless anyways).
	//- Other unknown characters have the width of a space.</p>
	int get_width(unsigned char letter) const;
	
	int get_width(
		const std::string &str,
		const CL_Size &max_size = CL_Size(0,0),
		const std::string &delim = std::string(" ")) const;
	
	int get_width(
		const std::string::const_iterator &start,
		const std::string::const_iterator &end,
		const CL_Size &max_size = CL_Size(0,0),
		const std::string &delim = std::string(" ")) const;
	
	//: Returns the drawn size of a string.
	//return: The size in pixels.
	//param letter: Character to get the size of. If unrecognized, returns space width.
	//param str: String to get the size of.
	//param start: A starting iterator, inclusive.
	//param end: An ending iterator, exclusive.
	//param max_size: Same effect as the size of the dest rectangle passed to draw(), for word wrapping and height truncating
	//param delim: When word-wrapping, specifies characters (other than newlines) that act as word seperators.
	//- <p> If all you want is the size, then its more efficient to call
	//- this function than to use test_draw.</p>
	CL_Size get_size(unsigned char letter) const;
	
	CL_Size get_size(
		const std::string &str,
		const CL_Size &max_size = CL_Size(0,0),
		const std::string &delim = std::string(" ")) const;
	
	CL_Size get_size(
		const std::string::const_iterator &start,
		const std::string::const_iterator &end,
		const CL_Size &max_size = CL_Size(0,0),
		const std::string &delim = std::string(" ")) const;
	
	//: Returns the glyphs as a CL_Sprite.
	const CL_Sprite &get_glyphs() const;
	
	//: Returns the glyph-index for a specific letter.
	//return: The frame number of the character's glyph, or -1 if there is no such character in the font.
	int get_glyph_index(unsigned char letter) const;
	
	//: Resource owning this sprite, if any.
	CL_Resource resource;
	
	//: Gets the return value of a draw() without drawing anything.
	//return: A CL_Font::PrintInfo, containing stats about the text that would've been drawn.
	//param str: The input string to process.
	//param start: String position to begin processing at, inclusive.
	//param end: String position to end processing at, exclusive.
	//param x, y: Anchor position to simulate draw at. Actual position depends on the alignment mode.
	//param dest: Rectangle to draw text in. The text will be word-wrapped against delimiters to fit inside the rectangle.
	//param delim: When word-wrapping, specifies characters (other than newlines) that act as word seperators.
	//- <p> You can specify a dest rectangle with a width or height of zero or less to disable word wrapping
	//- or height truncating, respectively. </p>
	CL_Font::PrintInfo test_draw(
		int x,
		int y,
		const std::string &str) const;
	
	CL_Font::PrintInfo test_draw(
		const CL_Rect &dest,
		const std::string &str,
		const std::string &delim = std::string(" ")) const;
	
	CL_Font::PrintInfo test_draw(
		int x,
		int y,
		const std::string::const_iterator &start,
		const std::string::const_iterator &end) const;
	
	CL_Font::PrintInfo test_draw(
		const CL_Rect &dest,
		const std::string::const_iterator &start,
		const std::string::const_iterator &end,
		const std::string &delim = std::string(" ")) const;
	
//! Operations:
public:
	//: Copy assignment operator.
	CL_Font &operator =(const CL_Font &copy);
	
	//: Draws text to a graphic context.
	//return: A CL_Font::PrintInfo, containing stats about the text that was drawn.
	//param str: The input string to draw.
	//param start: String position to begin drawing at, inclusive.
	//param end: String position to end drawing at, exclusive.
	//param x, y: Anchor position to draw at. Actual drawing position depends on the alignment mode.
	//param gc: Graphic context on which to draw. If null, will use CL_Display's current graphic context.
	//param dest: Rectangle to draw text in. The text will be word-wrapped against delimiters to fit inside the rectangle.
	//param delim: When word-wrapping, specifies characters (other than newlines) that act as word seperators.
	//- <p> You can specify a dest rectangle with a width or height of zero or less to disable word wrapping
	//- or height truncating, respectively. </p>
	CL_Font::PrintInfo draw(
		int x,
		int y,
		const std::string &str,
		CL_GraphicContext *context = 0);
	
	CL_Font::PrintInfo draw(
		const CL_Rect &dest,
		const std::string &str,
		const std::string &delim = std::string(" "),
		CL_GraphicContext *context = 0);
	
	CL_Font::PrintInfo draw(
		int x,
		int y,
		const std::string::const_iterator &start,
		const std::string::const_iterator &end,
		CL_GraphicContext *context = 0);
	
	CL_Font::PrintInfo draw(
		const CL_Rect &dest,
		const std::string::const_iterator &start,
		const std::string::const_iterator &end,
		const std::string &delim = std::string(" "),
		CL_GraphicContext *context = 0);
	
	//: Insert glyphs and the font into a CL_GlyphBuffer.
	//return: A CL_Font::PrintInfo, containing stats about the text that was inserted.
	//param str: The input string to draw.
	//param start: String position to begin drawing at, inclusive.
	//param end: String position to end drawing at, exclusive.
	//param x, y: Anchor position to draw at. Actual drawing position depends on the alignment mode.
	//param gb: Glyph buffer to insert glyphs/font into.
	//param dest: Rectangle to draw text in. The text will be word-wrapped against delimiters to fit inside the rectangle.
	//param delim: When word-wrapping, specifies characters (other than newlines) that act as word seperators.
	//- <p> You can specify a dest rectangle with a width or height of zero or less to disable word wrapping
	//- or height truncating, respectively. </p>
	CL_Font::PrintInfo draw_to_gb(
		int x,
		int y,
		const std::string &str,
		CL_GlyphBuffer &gb) const;
	
	CL_Font::PrintInfo draw_to_gb(
		const CL_Rect &dest,
		const std::string &str,
		CL_GlyphBuffer &gb,
		const std::string &delim = std::string(" ")) const;
	
	CL_Font::PrintInfo draw_to_gb(
		int x,
		int y,
		const std::string::const_iterator &start,
		const std::string::const_iterator &end,
		CL_GlyphBuffer &gb) const;
	
	CL_Font::PrintInfo draw_to_gb(
		const CL_Rect &dest,
		const std::string::const_iterator &start,
		const std::string::const_iterator &end,
		CL_GlyphBuffer &gb,
		const std::string &delim = std::string(" ")) const;
	
	//: Set width offset.
	//- <p> The width offset can be used to kern glyphs together or spread them apart. </p> 
	void set_width_offset(int offset);
	
	//: Set height offset.
	//- <p> The height offset can be used to create space between lines, or to merge them together. </p> 
	void set_height_offset(int offset);
	
	//: Set baseline.
	//- <p> The baseline determines the bottom of the font in cases where multiple fonts appear on one line.
	//- The bottom, in this case, excludes the bottoms of glyphs that stretch below the line, normally
	//- including characters such as lowercase 'g' and 'j'.</p>
	//- param baseline: Location of baseline as number of pixels up from bottom of glyph. </p>
	void set_baseline(int baseline);
	
	//: Set scale for x and y directions individually.
	//- <p> 1.0f is normal scale, 2.0f is twice the size, etc. </p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void set_scale(float x, float y);
	
	//: Sets transparency.
	//- <p> 0.0f is full transparency, and 1.0f is full visibility. </p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void set_alpha(float alpha);
	
	//: Sets the color.
	//- <p> Alpha 0.0f is full transparency, and 1.0f is full visibility (solid). </p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void set_color(float r, float g, float b, float a = 1.0f);
	
	//: Sets blending functions.
	//- <p> This property is set inside the glyphs sprite.</p>
	void set_blend_func(CL_BlendFunc src, CL_BlendFunc dest);
	
	//: Sets absolute rotation angle for individual glyphs.
	//- <p> This rotation setting affects each individual glyph, rather
	//- then a drawn text as a whole. The glyphs are merely rotated
	//- from their original positions, useful for spinny effects, but also usually
	//- making things unreadable.</p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void set_glyph_angle(float angle);
	
	//: Adds angle to current angle for individual glyphs.
	//- <p> This rotation setting affects each individual glyph, rather
	//- then a drawn text as a whole. The glyphs are merely rotated
	//- from their original positions, useful for spinny effects, but also usually
	//- making things unreadable.</p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void glyph_rotate(float angle);
	
	//: Sets glyph rotation hotspot.
	//- <p> This is the hotspot origin which determines the rotation hotspot
	//- for individual rotation of glyphs (see CL_Font::set_glyph_angle()).
	//- Total rotation (not the same as glyph rotation) of the resulting text depends on this hotspot being
	//- origin_top_left:0:0, so if you set it to anything else, the results
	//- of glyph rotation combined with total rotation are unreadable.
	//- This is fine if all you're doing is something like an explosion
	//- effect.</p>
	//- <p> This property is set inside the glyphs sprite.</p>
	void set_glyph_rotation_hotspot(CL_Origin origin, int x = 0, int y = 0);
	
	//: Sets absolute total rotation angle in degrees.
	//- <p> This only has an effect if you're using CL_Font's drawing mechanism itself.
	//- If the font is just being used inside a complex multi-font class (such as
	//- CL_TextStyler) then it makes no difference. If that's the case, use the 
	//- equivalent property of the class containing the font. </p>
	//- <p> Also note that rotation using this method with a glyph rotation hotspot
	//- besides origin_top_left:0:0 produces illegible results. </p>
	void set_angle(float angle);
	
	//: Adds angle in degrees to current total angle.
	//- <p> This only has an effect if you're using CL_Font's drawing mechanism itself.
	//- If the font is just being used inside a complex multi-font class (such as
	//- CL_TextStyler) then it makes no difference. If that's the case, use the 
	//- equivalent property of the class containing the font. </p>
	//- <p> Also note that rotation using this method with a glyph rotation hotspot
	//- besides origin_top_left:0:0 produces illegible results. </p>
	void rotate(float angle);
	
	//: Sets total rotation hotspot.
	//- <p> This only has an effect if you're using CL_Font's drawing mechanism itself.
	//- If the font is just being used inside a complex multi-font class (such as
	//- CL_TextStyler) then it makes no difference. If that's the case, use the 
	//- equivalent property of the class containing the font. </p>
	void set_rotation_hotspot(CL_Origin origin, int x = 0, int y = 0);
	
	//: Sets translation hotspot.
	//- <p> This only has an effect if you're using CL_Font's drawing mechanism itself.
	//- If the font is just being used inside a complex multi-font class (such as
	//- CL_TextStyler) then it makes no difference. If that's the case, use the 
	//- equivalent property of the class containing the font. </p>
	//- <p> The origin also serves as line justification, as with a word processor.
	//- That is, origin_*_center results in center justification, and so on. </p>
	void set_alignment(CL_Origin origin, int x = 0, int y = 0);
	
//! Implementation:
private:
	CL_Font_Generic *impl;
};

#endif
