/*
	$Id: sprite_description.h,v 1.20 2002/12/01 17:59:44 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanDisplay="Sprites"
//! header=display.h

#ifndef header_sprite_description
#define header_sprite_description

#ifdef WIN32
#pragma warning( disable : 4786)
#endif

#include <string>
#include <map>

#include "../Core/Math/rect.h"

class CL_ResourceManager;
class CL_PixelBuffer;
class CL_SpriteDescription_Generic;

//: This class contains everything to construct a sprite - its data, default settings etc.
//- <p>CL_SpriteDescription is used by CL_Sprite to construct itself based
//- on the description. The description class give a big flexibility
//- creating a sprite, individual frames can come from different image
//- sources, or be cut out from an image source using various different
//- techniques.</p>
class CL_SpriteDescription
{ 
//! Construction:
public:
	//: Constructs a sprite description.
	//param resource_id: Resource name of a sprite description resource.
	//param resources: Resource manager used to load resource.
	CL_SpriteDescription(const std::string &resource_id, CL_ResourceManager *resources);

	CL_SpriteDescription();

	CL_SpriteDescription(const CL_SpriteDescription &copy);

	~CL_SpriteDescription();

//! Attributes:
public:
	//: CL_PixelBuffer *provider, CL_Rect position.
	typedef std::pair<CL_PixelBuffer *, CL_Rect> FramePair;

	//: Returns a list over all available frames.
	const std::list<FramePair> &get_frames() const;

//! Operations:
public:
	//: Copy assignment operator.
	CL_SpriteDescription &operator =(const CL_SpriteDescription &copy);

	//: Adds a single image.
	//param provider: Image source.
	//param delete_provider: If true, deletes provider when sprite description is destroyed.
	void add_frame(CL_PixelBuffer *provider, bool delete_provider = false);

	//: Adds images formed in a grid.
	//- <p>This function will cut out a grid of frames from one image.</p>
	//param provider: Image source.
	//param xpos, ypos: Position of where image grid starts.
	//param width, height: Size of a frame in the grid.
	//param xarray, yarray: Number of columns and rows in grid.
	//param array_skipframes: Number of frames to skip at last gridline.
	//param xspacing, yspacing: Pixel interspacing between grid frames.
	//param delete_provider: If true, deletes provider when sprite description is destroyed.
	void add_gridclipped_frames(CL_PixelBuffer *provider, int xpos, int ypos, int width, int height, int xarray = 1, int yarray = 1, int array_skipframes = 0, int xspacing = 0, int yspacing = 0, bool delete_provider = false);

	//: Adds images separated with pure alpha (within trans_limit).
	//- <p>The alpha clipper will cut out frames from an image based on
	//- the transparency in the picture. It first determines the height
	//- of a row by searching for the first line that it considers
	//- completely transparent. Then it finds the width of each frame on
	//- this line by looking for columns that are completely transparency.</p>
	//param provider: Image source.
	//param xpos, ypos: Upper left position where alpha cutting should begin.
	//param trans_limit: Amount of non-transparent alpha allowed before a pixel is not considered transparent.
	//param delete_provider: If true, deletes provider when sprite description is destroyed.
	void add_alphaclipped_frames(CL_PixelBuffer *provider, int xpos = 0, int ypos = 0, float trans_limit = 0.05f, bool delete_provider = false);

	//: Adds images separated with palette-colours defining the boundaries.
	//param provider: Image source.
	//param xpos, ypos: Upper left position where cutting should begin.
	//param delete_provider: If true, deletes provider when sprite description is destroyed.
	void add_paletteclipped_frames(CL_PixelBuffer *provider, int xpos = 0, int ypos = 0, bool delete_provider = false);
	
//! Implementation:
private:
	//: SpriteDescription implementation.
	CL_SpriteDescription_Generic *impl;
};

#endif


