/*
	$Id: surface.h,v 1.37 2003/02/05 04:38:24 plasmoid Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanDisplay="Display 2D"
//! header=display.h

#ifndef header_surface
#define header_surface

#ifdef WIN32
#pragma warning( disable : 4786)
#endif

#include "blend_func.h"
#include "../Core/Math/origin.h"
#include <string>

#include "../Core/Resources/resource.h"

class CL_PixelBuffer;
class CL_GraphicContext;
class CL_Rect;
class CL_ResourceManager;
class CL_Surface_Generic;

//: Construct hardware accelerated (where available) 2D surface.
// TODO: Add flip_vertical, flip_horizontal ?
class CL_Surface
{
//! Enums:
public:
	enum Hint
	{
		hint_opaque,   // surface only contain opaque data (no transparency)
		hint_colorkey, // surface only has colorkey alpha (transparency)
		hint_alpha     // surface with full alpha support (translucency)
	};

//! Construction:
public:
	// TODO: Add some hint to specify 16 or 32 bit surface.
	// TODO: Add some hint to specify mipmapping or not.

	//: Constructs a surface.
	//param provider: Image that surface will use.
	//param delete_provider: If true, will delete provider after it has copied its image data.
	//param hint: Hints what sort of optimizations CL_Surface can perform to increase rendering performance.
	//param filename: Name of an image file to load image data from.
	CL_Surface();

	CL_Surface(
		CL_PixelBuffer *provider,
		bool delete_provider = false,
		Hint hint = hint_colorkey);

	CL_Surface(
		const std::string &resource_id,
		CL_ResourceManager *manager);

	CL_Surface(const std::string &filename, Hint hint = hint_colorkey);

	CL_Surface(const CL_Surface &copy);

	virtual ~CL_Surface();

//! Attributes:
public:
	//: Returns the width.
	int get_width() const;
	
	//: Returns the height.
	int get_height() const;
	
	//: Returns current angle in degrees.
	float get_angle() const;

	//: Returns scale for x and y.
	//- <p> 1.0f is normal scale, 2.0f is twice the size, etc. </p>
	void get_scale(float &x, float &y) const;

	//: Returns current alpha.
	//- <p> Alpha 0.0f is full transparency, and 1.0f is full visibility (solid). </p>
	float get_alpha() const;

	//: Returns current color.
	//- <p> Alpha 0.0f is full transparency, and 1.0f is full visibility (solid). </p>
	void get_color(float &red, float &green, float &blue, float &alpha);

	//: Returns blending functions.
	void get_blend_func(CL_BlendFunc &src, CL_BlendFunc &dest) const;

	//: Returns translation hotspot.
	void get_alignment(CL_Origin &origin, int &x, int &y) const;

	//: Returns rotation hotspot.
	void get_rotation_hotspot(CL_Origin &origin, int &x, int &y) const;

	//: Resource owning this surface, if any.
	CL_Resource resource;

//! Operations:
public:
	//: Copy assignment operator.
	CL_Surface &operator =(const CL_Surface &copy);

	//: Draw surface on graphic context.
	//param x, y: Anchor position of where to render surface. Actual rendering position depends on the anchor and the alignment mode.
	//param gc: Graphic context on which to render upon. If null, will use CL_Display's current graphic context.
	//param dest: Rectangle to draw surface in.
	//param src: Rectangle specifying the sub section of a surface to render.
	void draw(
		int x = 0,
		int y = 0,
		CL_GraphicContext *context = 0);

	void draw(
		const CL_Rect &dest,
		CL_GraphicContext *context = 0);

	void draw(
		const CL_Rect &src,
		const CL_Rect &dest,
		CL_GraphicContext *context = 0);

	//: Set absolute rotation angle in degrees.
	void set_angle(float angle);

	//: Add angle in degrees to current angle.
	void rotate(float angle);

	//: Set scale for x and y directions individually.
	//- <p> 1.0f is normal scale, 2.0f is twice the size, etc. </p>
	void set_scale(float x, float y);

	//: Sets transparency.
	//- <p> Alpha 0.0f is full transparency, and 1.0f is full visibility (solid). </p>
	void set_alpha(float alpha);

	//: Sets the color.
	//- <p> Alpha 0.0f is full transparency, and 1.0f is full visibility (solid). </p>
	void set_color(float r, float g, float b, float a = 1.0f);
	
	//: Sets blending functions.
	void set_blend_func(CL_BlendFunc src, CL_BlendFunc dest);

	//: Sets translation hotspot.
	void set_alignment(CL_Origin origin, int x = 0, int y = 0);

	//: Sets rotation hotspot.
	void set_rotation_hotspot(CL_Origin origin, int x = 0, int y = 0);

// Implementation:
private:
	CL_Surface_Generic *impl;
};

#endif
